class DuplicateDefineWarning(SyntaxWarning):
    pass


class InvalidLabelWarning(SyntaxWarning):
    pass


class FieldNotFoundWarning(SyntaxWarning):
    pass


class ClassHasDefinedWarning(SyntaxWarning):
    pass
